@@ -1,5 +1,5 @@
 /* URI.java -- An URI class
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -346,8 +346,15 @@ public final class URI
   private static String getURIGroup(Matcher match, int group)
   {
     String matched = match.group(group);
-    return matched.length() == 0 
-      ? ((match.group(group - 1).length() == 0) ? null : "") : matched;
+    if (matched == null || matched.length() == 0)
+      {
+	String prevMatched = match.group(group -1);
+	if (prevMatched == null || prevMatched.length() == 0)
+	  return null;
+	else
+	  return "";
+      }
+    return matched;
   }
 
   /**
