@@ -1045,7 +1045,7 @@ public class JList extends JComponent implements Accessible, Scrollable
    *
    * @param listData Initial data to populate the list with
    */
-  public JList(Vector listData)
+  public JList(Vector<?> listData)
   {
     init();
     setListData(listData);
@@ -1070,14 +1070,16 @@ public class JList extends JComponent implements Accessible, Scrollable
     layoutOrientation = VERTICAL;
     opaque = true;
     valueIsAdjusting = false;
-    visibleRowCount = 8;
+    visibleRowCount = 7;
 
     cellRenderer = new DefaultListCellRenderer();
     listListener = new ListListener();
 
     setModel(new DefaultListModel());
     setSelectionModel(createSelectionModel());
-
+    setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
+    setLayout(null);
+    
     updateUI();
   }
 
@@ -1255,14 +1257,17 @@ public class JList extends JComponent implements Accessible, Scrollable
    *
    * @param a A number in the half-open range <code>[0, x)</code> where
    * <code>x = getModel.getSize()</code>, indicating the index of an
-   * element in the list to select.
+   * element in the list to select. When &lt; 0 the selection is cleared.
    *
    * @see #setSelectionMode
    * @see #selectionModel
    */
   public void setSelectedIndex(int a)
   {
-    selectionModel.setSelectionInterval(a, a);
+    if (a < 0)
+      selectionModel.clearSelection();
+    else
+      selectionModel.setSelectionInterval(a, a);
   }
 
   /**
@@ -1322,7 +1327,11 @@ public class JList extends JComponent implements Accessible, Scrollable
 
 
   /**
-   * Returns index of the cell to which specified location is closest to
+   * Returns index of the cell to which specified location is closest to. If
+   * the location is outside the bounds of the list, then the greatest index
+   * in the list model is returned. If the list model is empty, then
+   * <code>-1</code> is returned.
+   *
    * @param location for which to look for in the list
    * 
    * @return index of the cell to which specified location is closest to.
@@ -1562,7 +1571,7 @@ public class JList extends JComponent implements Accessible, Scrollable
    * @param listData The object array to build a new list model on
    * @see #setModel
    */
-  public void setListData(final Vector listData)
+  public void setListData(final Vector<?> listData)
   {
     setModel(new AbstractListModel()
         {
@@ -1620,10 +1629,15 @@ public class JList extends JComponent implements Accessible, Scrollable
    * #listListener} is unsubscribed from the existing model, if it exists,
    * and re-subscribed to the new model.
    *
-   * @param model The new property value
+   * @param model  the new model (<code>null</code> not permitted).
+   * 
+   * @throws IllegalArgumentException if <code>model</code> is 
+   *         <code>null</code>.
    */
   public void setModel(ListModel model)
   {
+    if (model == null) 
+      throw new IllegalArgumentException("Null 'model' argument.");
     if (this.model == model)
       return;
     
