@@ -44,12 +44,14 @@ import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.event.MouseEvent;
 import java.io.Serializable;
+import java.util.Locale;
 import java.util.Vector;
 
 import javax.accessibility.Accessible;
 import javax.accessibility.AccessibleContext;
 import javax.accessibility.AccessibleRole;
 import javax.accessibility.AccessibleSelection;
+import javax.accessibility.AccessibleStateSet;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 import javax.swing.plaf.TabbedPaneUI;
@@ -136,7 +138,12 @@ public class JTabbedPane extends JComponent implements Serializable,
      */
     public Accessible getAccessibleChild(int i)
     {
-      return null;
+      // Testing shows that the reference implementation returns instances
+      // of page here.
+      Accessible child = null;
+      if (i >= 0 && i < tabs.size())
+        child = (Page) tabs.get(i);
+      return child;
     }
 
     /**
@@ -273,6 +280,8 @@ public class JTabbedPane extends JComponent implements Serializable,
    * A private class that holds all the information  for each tab.
    */
   private class Page
+    extends AccessibleContext
+    implements Accessible
   {
     /** The tooltip string. */
     private String tip;
@@ -337,9 +346,10 @@ public class JTabbedPane extends JComponent implements Serializable,
      */
     public void setComponent(Component c)
     {
-      remove(component);
-      this.component = c;
-      add(c);
+      int i = indexOfComponent(component);
+      insertTab(title, icon, c, tip, i);
+      component = c;
+      removeTabAt(i);
     }
 
     /**
@@ -496,7 +506,7 @@ public class JTabbedPane extends JComponent implements Serializable,
      */
     public int getMnemonic()
     {
-      return (int) mnemonicKey;
+      return mnemonicKey;
     }
 
     /**
@@ -552,6 +562,74 @@ public class JTabbedPane extends JComponent implements Serializable,
 
       underlinedChar = index;
     }
+
+    /**
+     * Returns the accessible context, which is this object itself.
+     *
+     * @return the accessible context, which is this object itself
+     */
+    public AccessibleContext getAccessibleContext()
+    {
+      return this;
+    }
+
+    /**
+     * Returns the accessible role of this tab, which is always
+     * {@link AccessibleRole#PAGE_TAB}.
+     *
+     * @return the accessible role of this tab
+     */
+    public AccessibleRole getAccessibleRole()
+    {
+      return AccessibleRole.PAGE_TAB;
+    }
+
+    public AccessibleStateSet getAccessibleStateSet()
+    {
+      // FIXME: Implement this properly.
+      return null;
+    }
+
+    public int getAccessibleIndexInParent()
+    {
+      // FIXME: Implement this properly.
+      return 0;
+    }
+
+    /**
+     * Returns the number of accessible children, which is always one (the
+     * component of this tab).
+     *
+     * @return the number of accessible children
+     */
+    public int getAccessibleChildrenCount()
+    {
+      return 1;
+    }
+
+    /**
+     * Returns the accessible child of this tab, which is the component
+     * displayed by the tab.
+     *
+     * @return the accessible child of this tab
+     */
+    public Accessible getAccessibleChild(int i)
+    {
+      // A quick test shows that this method always returns the component
+      // displayed by the tab, regardless of the index.
+      return (Accessible) component;
+    }
+
+    /**
+     * Returns the locale of this accessible object.
+     *
+     * @return the locale of this accessible object
+     */
+    public Locale getLocale()
+    {
+      // TODO: Is this ok?
+      return Locale.getDefault();
+    }
   }
 
   private static final long serialVersionUID = 1614381073220130939L;
@@ -591,9 +669,11 @@ public class JTabbedPane extends JComponent implements Serializable,
 
   /**
    * Creates a new JTabbedPane object using wrap tab layout  and the given
-   * tabPlacement.
+   * <code>tabPlacement</code>, where <code>tabPlacement</code> can be one
+   * of the following values: {@link #TOP}, {@link #BOTTOM}, {@link #LEFT} or
+   * {@link #RIGHT}.
    *
-   * @param tabPlacement Where the tabs will be placed.
+   * @param tabPlacement where the tabs will be placed
    */
   public JTabbedPane(int tabPlacement)
   {
@@ -601,11 +681,14 @@ public class JTabbedPane extends JComponent implements Serializable,
   }
 
   /**
-   * Creates a new JTabbedPane object with the given tabPlacement and
-   * tabLayoutPolicy.
+   * Creates a new JTabbedPane object with the given <code>tabPlacement</code>
+   * and <code>tabLayoutPolicy</code>. The <code>tabPlacement</code> can be one
+   * of the following values: {@link #TOP}, {@link #BOTTOM}, {@link #LEFT} or
+   * {@link #RIGHT}. The <code>tabLayoutPolicy</code> can be either
+   * {@link #SCROLL_TAB_LAYOUT} or {@link #WRAP_TAB_LAYOUT}.
    *
-   * @param tabPlacement Where the tabs will be placed.
-   * @param tabLayoutPolicy The way tabs will be placed.
+   * @param tabPlacement where the tabs will be placed
+   * @param tabLayoutPolicy the way tabs will be placed
    *
    * @throws IllegalArgumentException If tabLayoutPolicy or tabPlacement are
    *         not valid.
@@ -620,7 +703,7 @@ public class JTabbedPane extends JComponent implements Serializable,
       throw new IllegalArgumentException("tabLayoutPolicy is not valid.");
     this.tabPlacement = tabPlacement;
     layoutPolicy = tabLayoutPolicy;
-
+    
     changeEvent = new ChangeEvent(this);
     changeListener = createChangeListener();
 
@@ -912,7 +995,7 @@ public class JTabbedPane extends JComponent implements Serializable,
     if (getSelectedIndex() == -1)
       setSelectedIndex(0);
 
-    layout();
+    revalidate();
     repaint();
   }
 
@@ -968,6 +1051,7 @@ public class JTabbedPane extends JComponent implements Serializable,
       super.add(component);
     else
       insertTab(component.getName(), null, component, null, tabs.size());
+    
     return component;
   }
 
@@ -1051,45 +1135,37 @@ public class JTabbedPane extends JComponent implements Serializable,
   }
 
   /**
-   * The tab and it's associated component are removed. After the component
-   * has been removed from the JTabbedPane, it's set visible to ensure that
-   * it can be seen.
+   * Removes the tab at index. After the component associated with 
+   * index is removed, its visibility is reset to true to ensure it 
+   * will be visible if added to other containers.
    *
    * @param index The index of the tab to remove.
    */
   public void removeTabAt(int index)
   {
     checkIndex(index, 0, tabs.size());
-    Component c = getComponentAt(index);
-    super.remove(index);
-    c.show();
     tabs.remove(index);
+    getComponentAt(index).show();
   }
 
   /**
-   * This method removes the component from the JTabbedPane. After the
-   * component has been removed from the JTabbedPane, it's  set visible to
-   * ensure that it can be seen.
+   * Removes the specified Component from the JTabbedPane.
    *
    * @param component The Component to remove.
    */
   public void remove(Component component)
   {
-    // This simply removes the component.
-    int index = indexOfComponent(component);
     super.remove(component);
-    component.show();
-    setComponentAt(index, null);
   }
 
   /**
-   * This method removes the tab and component from the JTabbedPane. It simply
-   * calls removeTabAt(int index).
+   * Removes the tab and component which corresponds to the specified index.
    *
    * @param index The index of the tab to remove.
    */
   public void remove(int index)
   {
+    super.remove(index);
     removeTabAt(index);
   }
 
