@@ -38,10 +38,12 @@ exception statement from your version. */
 package java.lang;
 
 import java.lang.annotation.Annotation;
+import java.lang.reflect.Array;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.Field;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
+import java.lang.reflect.Modifier;
 import java.lang.reflect.Type;
 import java.lang.reflect.TypeVariable;
 
@@ -194,7 +196,7 @@ final class VMClass
    * @param ignoreInnerClassesAttrib if set, return the real modifiers, not
    * the ones specified in the InnerClasses attribute.
    * @return the modifiers of this class
-   * @see Modifer
+   * @see Modifier
    * @since 1.1
    */
   static native int getModifiers(Class klass, boolean ignoreInnerClassesAttrib);
@@ -213,7 +215,7 @@ final class VMClass
    * Like <code>getDeclaredClasses()</code> but without the security checks.
    *
    * @param klass the Class object that's calling us
-   * @param pulicOnly Only public classes should be returned
+   * @param publicOnly Only public classes should be returned
    */
   static native Class[] getDeclaredClasses(Class klass, boolean publicOnly);
 
@@ -221,7 +223,7 @@ final class VMClass
    * Like <code>getDeclaredFields()</code> but without the security checks.
    *
    * @param klass the Class object that's calling us
-   * @param pulicOnly Only public fields should be returned
+   * @param publicOnly Only public fields should be returned
    */
   static native Field[] getDeclaredFields(Class klass, boolean publicOnly);
 
@@ -229,7 +231,7 @@ final class VMClass
    * Like <code>getDeclaredMethods()</code> but without the security checks.
    *
    * @param klass the Class object that's calling us
-   * @param pulicOnly Only public methods should be returned
+   * @param publicOnly Only public methods should be returned
    */
   static native Method[] getDeclaredMethods(Class klass, boolean publicOnly);
 
@@ -238,7 +240,7 @@ final class VMClass
    * the security checks.
    *
    * @param klass the Class object that's calling us
-   * @param pulicOnly Only public constructors should be returned
+   * @param publicOnly Only public constructors should be returned
    */
   static native Constructor[] getDeclaredConstructors(Class klass, boolean publicOnly);
 
